@@ -6,6 +6,7 @@ package CHI::Driver::SharedMem;
 
 use warnings;
 use strict;
+use CHI::Constants qw(CHI_Meta_Namespace);
 use Moose;
 use IPC::SysV qw(S_IRUSR S_IWUSR IPC_CREAT);
 use IPC::SharedMem;
@@ -46,11 +47,11 @@ CHI::Driver::SharedMem - Cache data in shared memory
 
 =head1 VERSION
 
-Version 0.17
+Version 0.18
 
 =cut
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 # FIXME - get the pod documentation right so that the layout of the memory
 # area looks correct in the man page
@@ -68,7 +69,7 @@ See L<IPC::SharedMem> for more information.
     use CHI;
     my $cache = CHI->new(
 	driver => 'SharedMem',
-	max_size => 8 * 1024,	# Size of the cache
+	max_size => 2 * 1024,	# Size of the cache
 	shm_size => 32 * 1024,	# Size of the shared memory area
 	shm_key => 12344321,	# Choose something unique, but the same across
 				# all caches so that namespaces will be shared,
@@ -108,6 +109,9 @@ sub store {
 	$self->_lock(type => 'write');
 	my $h = $self->_data();
 	$h->{$self->namespace()}->{$key} = $value;
+	# if($self->{'is_size_aware'}) {
+		# $h->{CHI_Meta_Namespace()}->{'last_used_time'}->{$key} = time;
+	# }
 	$self->_data($h);
 	$self->_unlock();
 }
@@ -127,6 +131,12 @@ sub fetch {
 	# print $tulip __LINE__, "\n";
 	my $rc = $self->_data()->{$self->namespace()}->{$key};
 	# print $tulip __LINE__, "\n";
+	if($self->{is_size_aware}) {
+		$self->_lock(type => 'write');
+		my $h = $self->_data();
+		$h->{CHI_Meta_Namespace()}->{last_used_time}->{$key} = time;
+		$self->_data($h);
+	}
 	$self->_unlock();
 	# print $tulip __LINE__, "\n";
 	# close $tulip;
@@ -145,6 +155,7 @@ sub remove {
 	$self->_lock(type => 'write');
 	my $h = $self->_data();
 	delete $h->{$self->namespace()}->{$key};
+	delete $h->{CHI_Meta_Namespace()}->{last_used_time}->{$key};
 	$self->_data($h);
 	$self->_unlock();
 
@@ -203,6 +214,34 @@ sub get_namespaces {
 	return @rc;
 }
 
+=head2 default_discard_policy
+
+Use an LRU algorithm to discard items when the cache can't add anything
+
+=cut
+
+sub default_discard_policy { 'lru' }
+
+=head2 discard_policy_lru
+
+When the Shared memory area is getting close to full, discard the least recently used objects
+
+=cut
+
+sub discard_policy_lru {
+	# return;	# debugging why I get uninitialized values in the sort
+	my $self = shift;
+
+	$self->_lock(type => 'read');
+	my $last_used_time = $self->_data()->{CHI_Meta_Namespace()}->{last_used_time};
+	$self->_unlock();
+	my @keys_in_lru_order =
+		sort { $last_used_time->{$a} <=> $last_used_time->{$b} } $self->get_keys();
+	return sub {
+		shift(@keys_in_lru_order);
+	};
+}
+
 # Internal routines
 
 # The area must be locked by the caller
@@ -230,6 +269,8 @@ sub _build_shm {
 }
 
 sub _build_lock {
+	return;
+
 	my $self = shift;
 
 	# open(my $fd, '<', $0) || croak("$0: $!");
@@ -305,7 +346,7 @@ sub _data {
 		my $f = JSON::MaybeXS->new()->ascii()->encode($h);
 		my $cur_size = length($f);
 		# print $tulip __LINE__, "cmp $cur_size > ", $self->size(), "\n";
-		if($cur_size > $self->shm_size()) {
+		if($cur_size > ($self->shm_size() - $Config{intsize})) {
 			croak("sharedmem set failed - value too large? ($cur_size bytes) > ", $self->shm_size());
 		}
 		$self->shm()->write($f, $Config{intsize}, $cur_size);
@@ -318,7 +359,16 @@ sub _data {
 	# print $tulip "get: $cur_size bytes\n";
 	# close $tulip;
 	if($cur_size) {
-		return JSON::MaybeXS->new()->ascii()->decode($self->shm()->read($Config{intsize}, $cur_size));
+		my $rc;
+		eval {
+			$rc = JSON::MaybeXS->new()->ascii()->decode($self->shm()->read($Config{intsize}, $cur_size));
+		};
+		if($@) {
+			$self->_data_size(0);
+			croak($@);
+		}
+		return $rc;
+		# return JSON::MaybeXS->new()->ascii()->decode($self->shm()->read($Config{intsize}, $cur_size));
 	}
 	return {};
 }
